<?php
declare(strict_types=1);

namespace Flat101\MultiWidget\Controller\Adminhtml;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Catalog\Model\ImageUploader;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Filesystem;
use Magento\Framework\Filesystem\Io\File;
use Magento\Framework\UrlInterface;
use Magento\Store\Model\StoreManagerInterface;

abstract class Uploader extends Action implements HttpPostActionInterface
{
    const CUSTOM_IMAGE_PATH_TMP = 'multi_widget/tmp/images/';

    /** @var ImageUploader */
    protected $imageUploader;

    /** @var Filesystem */
    protected $filesystem;

    /** @var File */
    protected $fileIo;

    /** @var StoreManagerInterface */
    protected $storeManager;

    /**
     * Upload constructor.
     *
     * @param ImageUploader $imageUploader
     * @param Filesystem $filesystem
     * @param File $fileIo
     * @param StoreManagerInterface $storeManager
     * @param Context $context
     */
    public function __construct(
        ImageUploader $imageUploader,
        Filesystem $filesystem,
        File $fileIo,
        StoreManagerInterface $storeManager,
        Context $context
    ) {
        $this->imageUploader = $imageUploader;
        $this->filesystem = $filesystem;
        $this->fileIo = $fileIo;
        $this->storeManager = $storeManager;
        parent::__construct($context);
    }

    /**
     * @param string $imageUploadId
     * @return array
     */
    protected function getFileAttributes(string $imageUploadId): array
    {
        $fileAttributes = [];
        preg_match("/^(.*?)\[(.*?)\]\[(.*?)\]$/", $imageUploadId, $matches);
        $containerName = $matches[1];
        $rowId         = $matches[2];
        $fieldName     = $matches[3];
        if ($_FILES[$containerName]) {
            foreach ($_FILES[$containerName] as $attributeName => $rows) {
                if ($rows[$rowId] && $rows[$rowId][$fieldName]) {
                    $fileAttributes[$attributeName] = $rows[$rowId][$fieldName];
                }
            }
        }

        return $fileAttributes;
    }

    /**
     * @param $image
     * @param $destinationFolder
     * @return array
     */
    protected function manageFileUpload($image, $destinationFolder): array
    {
        try {
            $tmpPath = $this->filesystem->getDirectoryRead(DirectoryList::MEDIA)->getAbsolutePath() . self::CUSTOM_IMAGE_PATH_TMP;
            $destinationPath = $this->filesystem->getDirectoryRead(DirectoryList::MEDIA)->getAbsolutePath() . $destinationFolder;

            $fileAttributes = $this->getFileAttributes($image);
            $this->imageUploader->setBaseTmpPath($tmpPath);
            $this->imageUploader->setBasePath($destinationPath);
            $imageResult = $this->imageUploader->saveFileToTmpDir($fileAttributes);
            $imageName = $imageResult['name'];

            if (!is_dir($destinationPath)) {
                $this->fileIo->mkdir($destinationPath, 0775);
            }

            // Set image name with new name, If image already exist
            $newImageName = $this->updateImageName($destinationPath, $imageName);
            $this->fileIo->mv($tmpPath . $imageName, $destinationPath . $newImageName);
            $imageResult['cookie'] = [
                'name' => $this->_getSession()->getName(),
                'value' => $this->_getSession()->getSessionId(),
                'lifetime' => $this->_getSession()->getCookieLifetime(),
                'path' => $this->_getSession()->getCookiePath(),
                'domain' => $this->_getSession()->getCookieDomain(),
            ];

            $mediaUrl = $this->storeManager->getStore()->getBaseUrl(UrlInterface::URL_TYPE_MEDIA);
            $imageResult['name'] = $newImageName;
            $imageResult['file'] = $newImageName;
            $imageResult['url'] = $mediaUrl . $destinationFolder . $newImageName;

            return $imageResult;
        } catch (\Exception $e) {
            return ['error' => $e->getMessage(), 'errorcode' => $e->getCode()];
        }
    }

    /**
     * @param string $path
     * @param string $file_name
     * @return string
     */
    protected function updateImageName(string $path, string $file_name): string
    {
        if ($position = strrpos($file_name, '.')) {
            $name = substr($file_name, 0, $position);
            $extension = substr($file_name, $position);
        } else {
            $name = $file_name;
        }
        $new_file_path = $path . '/' . $file_name;
        $new_file_name = $file_name;
        $count = 0;
        while (file_exists($new_file_path)) {
            $new_file_name = $name . '_' . $count . $extension;
            $new_file_path = $path . '/' . $new_file_name;
            $count++;
        }

        return $new_file_name;
    }
}
