<?php
declare(strict_types=1);

namespace Flat101\MultiWidget\Model;

use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Api\ExtensionAttribute\JoinProcessorInterface;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Flat101\MultiWidget\Api\Data\MultiWidgetSearchResultsInterface;
use Flat101\MultiWidget\Api\Data\MultiWidgetSearchResultsInterfaceFactory;
use Flat101\MultiWidget\Api\Data\ItemsInterface;
use Flat101\MultiWidget\Api\ItemsRepositoryInterface;
use Flat101\MultiWidget\Api\Data\ItemsSearchResultsInterface;
use Flat101\MultiWidget\Model\ResourceModel\MultiWidget\CollectionFactory as MultiWidgetCollectionFactory;
use Flat101\MultiWidget\Api\Data\MultiWidgetInterface;
use Flat101\MultiWidget\Api\MultiWidgetRepositoryInterface;
use Flat101\MultiWidget\Model\ResourceModel\MultiWidget as ResourceModelMultiWidget;
use Magento\Framework\Api\SortOrderBuilder;

use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Api\FilterBuilder;
class MultiWidgetRepository implements MultiWidgetRepositoryInterface
{
    /** @var MultiWidgetFactory */
    private MultiWidgetFactory $multiWidgetFactory;

    /** @var ResourceModelMultiWidget */
    private ResourceModelMultiWidget $resourceModelMultiWidget;

    /** @var MultiWidgetCollectionFactory */
    private MultiWidgetCollectionFactory $multiWidgetCollectionFactory;

    /** @var ItemsRepositoryInterface */
    private ItemsRepositoryInterface $itemsRepository;

    /** @var MultiWidgetSearchResultsInterfaceFactory */
    private MultiWidgetSearchResultsInterfaceFactory $multiWidgetSearchResultsInterfaceFactory;

    /** @var CollectionProcessorInterface */
    private CollectionProcessorInterface $collectionProcessor;

    /** @var JoinProcessorInterface */
    private JoinProcessorInterface $extensionAttributesJoinProcessor;

    /** @var SearchCriteriaBuilder */
    private SearchCriteriaBuilder $searchCriteriaBuilder;

    /** @var SortOrderBuilder */
    private SortOrderBuilder $sortOrderBuilder;

    /** @var FilterBuilder */
    private FilterBuilder $filterBuilder;

    /**
     * TypesRepository constructor.
     *
     * @param MultiWidgetFactory $multiWidgetFactory
     * @param ResourceModelMultiWidget $resourceModelMultiWidget
     * @param MultiWidgetCollectionFactory $multiWidgetCollectionFactory
     * @param ItemsRepositoryInterface $itemsRepository
     * @param MultiWidgetSearchResultsInterfaceFactory $multiWidgetSearchResultsInterfaceFactory
     * @param CollectionProcessorInterface $collectionProcessor
     * @param JoinProcessorInterface $extensionAttributesJoinProcessor
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param FilterBuilder $filterBuilder
     * @param SortOrderBuilder $sortOrderBuilder
     */
    public function __construct(
        MultiWidgetFactory $multiWidgetFactory,
        ResourceModelMultiWidget $resourceModelMultiWidget,
        MultiWidgetCollectionFactory $multiWidgetCollectionFactory,
        ItemsRepositoryInterface $itemsRepository,
        MultiWidgetSearchResultsInterfaceFactory $multiWidgetSearchResultsInterfaceFactory,
        CollectionProcessorInterface $collectionProcessor,
        JoinProcessorInterface $extensionAttributesJoinProcessor,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        FilterBuilder $filterBuilder,
        SortOrderBuilder $sortOrderBuilder
    ) {
        $this->multiWidgetFactory                       = $multiWidgetFactory;
        $this->resourceModelMultiWidget                 = $resourceModelMultiWidget;
        $this->multiWidgetCollectionFactory             = $multiWidgetCollectionFactory;
        $this->itemsRepository                          = $itemsRepository;
        $this->multiWidgetSearchResultsInterfaceFactory = $multiWidgetSearchResultsInterfaceFactory;
        $this->collectionProcessor                      = $collectionProcessor;
        $this->extensionAttributesJoinProcessor         = $extensionAttributesJoinProcessor;
        $this->searchCriteriaBuilder                    = $searchCriteriaBuilder;
        $this->filterBuilder                            = $filterBuilder;
        $this->sortOrderBuilder                         = $sortOrderBuilder;
    }

    /**
     * @inheritdoc
     */
    public function save(MultiWidgetInterface $multiWidget): MultiWidgetInterface
    {
        $this->resourceModelMultiWidget->save($multiWidget);

        return $multiWidget;
    }

    /**
     * @inheritdoc
     */
    public function getById(int $entityId): MultiWidgetInterface
    {
        return $this->get($entityId);
    }

    /**
     * @inheritdoc
     */
    public function get($value, ?string $attributeCode = null): MultiWidgetInterface
    {
        /** @var MultiWidget $multiWidget */
        $multiWidget = $this->multiWidgetFactory->create()->load($value, $attributeCode);

        if (!$multiWidget->getId()) {
            throw new NoSuchEntityException(__('Unable to find Multi Widget'));
        }

        return $multiWidget;
    }

    /**
     * @inheritdoc
     */
    public function delete(MultiWidgetInterface $multiWidget): bool
    {
        $entityId = $multiWidget->getId();
        try {
            $this->resourceModelMultiWidget->delete($multiWidget);
        } catch (\Exception $e) {
            throw new CouldNotDeleteException(
                __('Unable to remove Multi Widget %1', $entityId)
            );
        }

        return true;
    }

    /**
     * @inheritdoc
     */
    public function deleteById(int $entityId): bool
    {
        $entity = $this->getById($entityId);

        return $this->delete($entity);
    }

    /**
     * @inheritdoc
     * @throws LocalizedException
     */
    public function getList(SearchCriteriaInterface $searchCriteria): MultiWidgetSearchResultsInterface
    {
        try {
            $collection = $this->multiWidgetCollectionFactory->create();
            $this->extensionAttributesJoinProcessor->process(
                $collection,
                MultiWidgetInterface::class
            );

            $this->collectionProcessor->process($searchCriteria, $collection);

            $searchResults = $this->multiWidgetSearchResultsInterfaceFactory->create();
            $searchResults->setItems($collection->getItems());
            $searchResults->setSearchCriteria($searchCriteria);
            $searchResults->setTotalCount($collection->getSize());

            return $searchResults;
        } catch (\Exception $exception) {
            throw new LocalizedException(__('Could not retrieve slides.'), $exception);
        }
    }

    /**
     * @inheritdoc
     */
    public function getItems(MultiWidgetInterface $multiWidget, bool $onlyEnabled = false): ItemsSearchResultsInterface
    {
//        $itemsCollection = $this->itemsCollectionFactory->create();
//        $itemsCollection->addFieldToFilter(ItemsInterface::MULTI_WIDGET_ID, $multiWidget->getId());
//
//        return $itemsCollection;

        if ($onlyEnabled) {
            $isActiveFilter = $this->filterBuilder->setField(ItemsInterface::STATUS)
                ->setConditionType('eq')
                ->setValue(1)
                ->create();

            $this->searchCriteriaBuilder->addFilters([$isActiveFilter]);
        }

        $multiWidgetIdFilter = $this->filterBuilder->setField(ItemsInterface::MULTI_WIDGET_ID)
            ->setConditionType('eq')
            ->setValue($multiWidget->getId())
            ->create();
        $this->searchCriteriaBuilder->addFilters([$multiWidgetIdFilter]);


        $sortOrder = $this->sortOrderBuilder->setField(ItemsInterface::POSITION)->setDirection('ASC')->create();
        $this->searchCriteriaBuilder->setSortOrders([$sortOrder]);

        $searchCriteria = $this->searchCriteriaBuilder->create();
        return $this->itemsRepository->getList($searchCriteria);
    }

    /**
     * @inheritdoc
     */
    public function getItemsById(int $entityId, bool $onlyEnabled = false): ItemsSearchResultsInterface
    {
        $entity = $this->getById($entityId);

        return $this->getItems($entity);
    }
}
